function [u,u_dot] = DSFDDLoc_CFS_Knownfo(f,f_dot,c,fo,S,Q,no_of_iter)
% [u,u_dot] = DSFDDLoc_CFS_Knownfo(f,f_dot,c,fo,S,Q,no_of_iter)
%
% Closed-form solution for DSF and DD localization when the center frequency is known.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement
% c:            scalar, signal propagation speed
% fo:           scalar, center frequency
% S:            (3 x M), sensor poition matrix, M=num. of sensors
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
% no_of_iter:   scalar, number of iterations in the first stage to update the weighting matrix
%
% Output:
% u:         (3 x 1), object position estimate
% u_dot:     (3 x 1), object velocity estimate
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors

% -- first stage --
p = (f-fo)*c/fo;
q = f_dot*c/fo;
Q = Q/Q(1,1);
h1 = [zeros(M,1);p.^2];
G1 = [S',-ones(M,1),zeros(M,1),-diag(p);
      zeros(M,K+1),ones(M,1),diag(q)];
% -- use SVD for LS estimation --
H1 = G1;
g1 = h1;
[U1,S1,V1] = svd(H1);
L1 = M+K+2;
iS1 = [diag( 1./diag(S1(1:L1,1:L1)) ),zeros(L1,2*M-L1)];
theta1 = V1*iS1*U1'*g1;
for ii = 1:1:no_of_iter
    r = theta1(K+3:end) ;
    B1 = [diag(r),zeros(M,M);
          diag(2*p),-diag(r)];
    % -- use SVD for WLS estimation --
    H1 = Q^(-0.5)/B1*G1;
    g1 = Q^(-0.5)/B1*h1;
    [U1,S1,V1] = svd(H1);
    L1 = M+K+2;
    iS1 = [diag( 1./diag(S1(1:L1,1:L1)) ),zeros(L1,2*M-L1)];
    theta1 = V1*iS1*U1'*g1; % first stage solution
end

% -- second stage --
sqS = diag(S'*S);
h2 = [theta1(1:K+2);theta1(K+3:end).^2 - sqS];
G2 = [zeros(K+2,K),eye(K+2),zeros(K+2,1);
     -2*S' zeros(M,K+2),ones(M,1)];
B2 = diag([ones(K+2,1);2*r]);
% -- use SVD for WLS estimation --
H2 = H1/B2*G2;
g2 = H1/B2*h2;
[U2,S2,V2] = svd(H2);
L2 = 2*K+3;
iS2 = [diag( 1./diag(S2(1:L2,1:L2)) ),zeros(L2,2*M-L2)];
theta2 = V2*iS2*U2'*g2; % second stage solution

% -- final stage --
u = theta2(1:K);
u_dot = theta2(K+1:2*K);
h3 = theta2;
G3 = zeros(2*K+3,2*K);
G3(1:2*K,1:2*K) = eye(2*K);
G3(2*K+1,1:2*K) = [0.5*u_dot',0.5*u'];
G3(2*K+2,1:2*K) = [zeros(1,K),u_dot'];
G3(2*K+3,1:2*K) = [u',zeros(1,K)];
B3 = zeros(2*K+3,2*K+3);
B3(1:2*K,1:2*K) = eye(2*K);
B3(2*K+1,1:2*K) = [-0.5*u_dot',-0.5*u'];
B3(2*K+2,1:2*K) = [zeros(1,K),-u_dot'];
B3(2*K+3,1:2*K) = [-u',zeros(1,K)];
B3(2*K+1,2*K+1) = 1;
B3(2*K+2,2*K+2) = 1;
B3(2*K+3,2*K+3) = 1;
% -- use SVD for WLS estimation --
H3 = H2/B3*G3;
g3 = H2/B3*h3;
[U3,S3,V3] = svd(H3);
L3 = 2*K;
iS3 = [diag( 1./diag(S3(1:L3,1:L3)) ),zeros(L3,2*M-L3)];
phi = V3*iS3*U3'*g3; % final solution
u = real( phi(1:K) );
u_dot = real( phi(K+1:2*K) );

end