function [u,u_dot,fo] = DSFDDLoc_CFS_Unknownfo(f,f_dot,c,S,Q,no_of_itar)
% [u,u_dot,fo] = DSFDDLoc_CFS_Unknownfo(f,f_dot,c,S,Q,no_of_itar)
%
% Closed-form solution for DSF and DD localization when the center frequency is unknown.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement
% c:            scalar, signal propagation speed
% S:            (3 x M), sensor poition matrix, M=num. of sensors
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
% no_of_iter:   scalar, number of iterations in the first stage to update the weighting matrix
%
% Output:
% u:         (3 x 1), object position estimate
% u_dot:     (3 x 1), object velocity estimate
% fo:        scalar, center frequency estimate
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors
tilde_fo = sum(f)/M; % self-generated hypothetical source frequency

% -- first stage --
tilde_f = f-tilde_fo;
sqS = diag(S'*S);
d1 = [tilde_f.^3; tilde_f.^2];
A1 = [  f_dot.*S',   3*tilde_f.^2,    -tilde_f,   ones(M,1), -f_dot,  zeros(M,M);
      zeros(M,K),   2*tilde_f,  -ones(M,1), zeros(M,2),  eye(M)];
% -- use SVD for WLS estimation --
H1 = Q^(-0.5)*A1;
g1 = Q^(-0.5)*d1;
[U1,S1,V1] = svd(H1);
L1 = M+K+4;
iS1 = [diag( 1./diag(S1(1:L1,1:L1)) ),zeros(L1,2*M-L1)];
beta_1 = V1*iS1*U1'*g1; 
fo = beta_1(K+1)+tilde_fo;
for ii = 1:1:no_of_itar
    C1 = [diag(2*(f-fo).^2+beta_1(K+5:end)-2*beta_1(K+1)^2),diag(beta_1(K+4)-S'*beta_1(1:K));
        2*diag(tilde_f-beta_1(K+1)),zeros(M,M)];
    % -- use SVD for WLS estimation --
    H1 = Q^(-0.5)/C1*A1;
    g1 = Q^(-0.5)/C1*d1;
    [U1,S1,V1] = svd(H1);
    L1 = M+K+4;
    iS1 = [diag( 1./diag(S1(1:L1,1:L1)) ),zeros(L1,2*M-L1)];
    beta_1 = V1*iS1*U1'*g1; % first stage solution
end

% -- second stage --
tilde_a = (beta_1(K+5:end)-beta_1(K+2)+beta_1(K+1)^2);
tilde_b = (S'*beta_1(1:K)-beta_1(K+4));
d2 = [beta_1(1:K+4);(tilde_fo/c)^2*tilde_a.*sqS];
A2 = [zeros(K+1,K),eye(K+1),zeros(K+1,4);
      zeros(1,2*K+1),3,-1,0,0;
      zeros(1,2*K+1),beta_1(K+1),-beta_1(K+1),0,0;
      zeros(1,2*K+3),1,0;
      2*tilde_a.*S',tilde_b.*S',-2*tilde_fo/(c^2)*tilde_a.*sqS,-tilde_a.*sqS/(c^2),zeros(M,1),-tilde_b,-tilde_a];
C2 = [eye(K+2),zeros(K+2,M+2);
      zeros(1,K),2*beta_1(K+1)^2-beta_1(K+2),0,1,zeros(1,M+1);
      zeros(1,K+3),1,zeros(1,M);
      -tilde_b.*S',2*beta_1(K+1)*(tilde_b.^2)./tilde_a,-(tilde_b.^2)./tilde_a,zeros(M,1),tilde_b,diag((tilde_b.^2)./tilde_a)];
% -- use SVD for WLS estimation --
H2 = H1/C2*A2;
g2 = H1/C2*d2;
[U2,S2,V2] = svd(H2);
L2 = 2*K+5;
iS2 = [diag( 1./diag(S2(1:L2,1:L2)) ),zeros(L2,2*M-L2)];
beta2 = V2*iS2*U2'*g2;  % second stage solution

% -- final stage --
fo = beta2(2*K+1)+tilde_fo;
u = beta2(1:K)*(c/fo)^2;
u_dot = beta2(K+1:2*K)*(c/fo)^2;
d3 = beta2;
A3 = [(fo/c)^2*eye(2*K),zeros(2*K,1);
      zeros(1,2*K),1;
      zeros(1,2*K),beta2(2*K+1);
      zeros(1,K), beta2(K+1:2*K)',0;
      0.5*beta2(K+1:2*K)',0.5*beta2(1:K)',0;
       beta2(1:K)',zeros(1,K+1)];
C3 = [eye(K),zeros(K,K),-2*beta2(1:K)/fo,zeros(K,4);
      zeros(K,K),eye(K),-2*beta2(K+1:2*K)/fo,zeros(K,4);
      zeros(1,2*K),1,zeros(1,4);
      zeros(1,2*K),-beta2(2*K+1),1,zeros(1,3);
      zeros(1,K),-u_dot',0,0,1,0,0;
      -0.5*u_dot',-0.5*u',0,0,0,1,0;
      -u',zeros(1,K+4),1];  
% - use SVD for WLS estimation --
H3 = H2/C3*A3;
g3 = H2/C3*d3;
[U3,S3,V3] = svd(H3);
L3 = 2*K+1;
iS3 = [diag( 1./diag(S3(1:L3,1:L3)) ),zeros(L3,2*M-L3)];
theta = V3*iS3*U3'*g3;

u = theta(1:K);
u_dot = theta(K+1:2*K); 
fo = (theta(2*K+1)+tilde_fo);

end



