function CRLB = DSFDDLoc_CRLB(fo,c,u,u_dot,S,Q,Idx_dsf,Idx_fo)
% CRLB = DSFDDLoc_CRLB(fo,c,u,u_dot,S,Q,Idx_dsf,Idx_fo)
%
% CRLB for DSF and DD localization.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% fo:       scalar, center frequency
% c:        scalar, signal propagation speed
% u:        (3 x 1), object position
% u_dot:    (3 x 1), object velocity
% S:        (3 x M), sensor poition matrix, M=num. of sensors
% Q:        (2M x 2M), covariance matrix of DSF and DD measurements
% Idx_dsf:   scalar, set Idx_dsf=1 for CRLB using DSF only,
%            set Idx_dsf = 2 for CRLB using both DSF and DD
% Idx_fo:    scalar, set Idx_fo=1 for known center frequency case,
%            set Idx_fo=0 for unknown center frequency case
%
% Output:
% CRLB:  (6 x 6), CRLB for object position and object velocity when center
%        frequency is known (Idx_fo =1), or 
%        (7 x 7), CRLB for object position, object velocity and center
%        frequency when center frequency is unknown (Idx_fo =0).
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of receivers
r = zeros(M,1);
f = zeros(M,1);
Jf_u = zeros(M,K);
Jf_udot = zeros(M,K);
Jfdot_u = zeros(M,K);
Jfdot_udot = zeros(M,K);
Jf_fo = zeros(M,1);
Jfdot_fo = zeros(M,1);
for i=1:1:M
    r(i) = norm(u - S(:,i));
    f(i) = fo*(1 - (u-S(:,i))'*u_dot/(c*r(i)) );
    Jf_u(i,1:K) = fo*( ((u-S(:,i))'*u_dot)/(c*r(i)^3)*(u-S(:,i))'- u_dot'/(c*r(i)) );
    Jf_udot(i,1:K) = -fo*(u-S(:,i))'/(c*r(i));
    Jfdot_u(i,1:K) = -(u-S(:,i))'/(c*r(i)^3)*( (f(i)-fo)^2*c^2/fo - fo*norm(u_dot)^2 )+1/(c*r(i))*( 2*(f(i)-fo)*c^2/fo*Jf_u(i,:) );
    Jfdot_udot(i,1:K) = 2/(c*r(i))*( ((f(i)-fo)*c^2)/fo*Jf_udot(i,:) - fo*u_dot' );  
    Jf_fo(i) = 1 - (u-S(:,i))'*u_dot/(c*r(i)) ;
    Jfdot_fo(i,1) = ( ((f(i)-fo)*c/fo)^2 - norm(u_dot)^2 )/(c*r(i));
end
if Idx_dsf==1
    Jm_phi = [Jf_u,Jf_udot];
    Jm_fo = Jf_fo;
    Q = Q(1:M,1:M);
else
    Jm_phi = [Jf_u,Jf_udot;Jfdot_u,Jfdot_udot];
    Jm_fo =  [Jf_fo;Jfdot_fo];
end
Jm_theta = [Jm_phi,Jm_fo];
if Idx_fo ==1
    FIM = Jm_phi'*inv(Q)*Jm_phi;
else
    FIM = Jm_theta'*inv(Q)*Jm_theta;
end

CRLB  = inv(FIM);

end

