function [u,u_dot] = DSFDDLoc_MLE_Knownfo(f,f_dot,c,fo,S,Q,phi0)
% [u,u_dot] = DSFDDLoc_MLE_Knownfo(f,f_dot,c,fo,S,Q,phi0)
%
% Gauss-Newton MLE solution for DSF and DD localization when the center frequency is known.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement
% c:            scalar, signal propagation speed
% fo:           scalar, center frequency
% S:            (3 x M), sensor poition matrix, M=num. of sensors
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
% phi0:         (6 x 1), initial solution of object position and object velocity
%
% Output:
% u:         (3 x 1), object position estimate
% u_dot:     (3 x 1), object velocity estimate
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors
IterMax = 30; % maximum number of iterations
phi = phi0;
for k=1:1:IterMax
    r = zeros(M,1);
    hat_f = zeros(M,1);
    hat_f_dot = zeros(M,1);
    Jf_u = zeros(M,K);
    Jf_udot = zeros(M,K);
    Jfdot_u = zeros(M,K);
    Jfdot_udot = zeros(M,K);
    u = phi(1:K,1);
    u_dot = phi(K+1:2*K,1);
    for i=1:1:M
        r(i) = norm(u - S(:,i));
        hat_f(i) = fo*(1 - (u-S(:,i))'*u_dot/(c*r(i)) );
        hat_f_dot(i) =  ( ( ((hat_f(i)-fo)*c )^2 )/fo - fo*norm(u_dot)^2 )/(c*r(i));
        Jf_u(i,1:K) = fo*( ((u-S(:,i))'*u_dot)/(c*r(i)^3)*(u-S(:,i))'- u_dot'/(c*r(i)) );
        Jf_udot(i,1:K) = -fo*(u-S(:,i))'/(c*r(i));
        Jfdot_u(i,1:K) = -(u-S(:,i))'/(c*r(i)^3)*( (hat_f(i)-fo)^2*c^2/fo - fo*norm(u_dot)^2 )+1/(c*r(i))*( 2*(hat_f(i)-fo)*c^2/fo*Jf_u(i,:) );
        Jfdot_udot(i,1:K) = 2/(c*r(i))*( ((hat_f(i)-fo)*c^2)/fo*Jf_udot(i,:) - fo*u_dot' );
    end
    Jm_phi = [Jf_u,Jf_udot;Jfdot_u,Jfdot_udot];
    delta_phi = 0.5*inv(Jm_phi'*inv(Q)*Jm_phi)*Jm_phi'*inv(Q)*([hat_f;hat_f_dot]-[f;f_dot]);
    phi = phi - delta_phi;
    if norm(delta_phi)<1e-6
        break;
    end
end

end

