function [u,u_dot,fo] = DSFDDLoc_MLE_Unknownfo(f,f_dot,c,S,Q,theta0)
% [u,u_dot,fo] = DSFDDLoc_MLE_Unknownfo(f,f_dot,c,S,Q,theta0)
%
% Gauss-Newton MLE solution for DSF and DD localization when the center frequency is unknown.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement,
% c:            scalar, signal propagation speed
% S:            (3 x M), sensor poition matrix, M=num. of sensors
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
% theta0:       (7 x 1), initial solution of object position, object velocity and center frequency
%
% Output:
% u:         (3 x 1), object position estimate
% u_dot:     (3 x 1), object velocity estimate
% fo:        scalar, center frequency estimate
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors
IterMax = 30; % maximum number of iterations
theta = theta0;
for k=1:1:IterMax
    r = zeros(M,1);
    hat_f = zeros(M,1);
    hat_f_dot = zeros(M,1);
    Jf_u = zeros(M,K);
    Jf_udot = zeros(M,K);
    Jf_fo = zeros(M,1);
    Jfdot_u = zeros(M,1);
    Jfdot_udot = zeros(M,K);
    Jfdot_fo = zeros(M,K);
    u = theta(1:K,1);
    u_dot = theta(K+1:2*K,1);
    fo = theta(2*K+1,1);
    for i=1:1:M
        r(i) = norm(u - S(:,i));
        hat_f(i) = fo*(1 - (u-S(:,i))'*u_dot/(c*r(i)) );
        hat_f_dot(i) =  ( ( ((hat_f(i)-fo)*c )^2 )/fo - fo*norm(u_dot)^2 )/(c*r(i));
        Jf_u(i,1:K) = fo*( ((u-S(:,i))'*u_dot)/(c*r(i)^3)*(u-S(:,i))'- u_dot'/(c*r(i)) );
        Jf_udot(i,1:K) = -fo*(u-S(:,i))'/(c*r(i));
        Jfdot_udot(i,1:K) = -(u-S(:,i))'/(c*r(i)^3)*( (hat_f(i)-fo)^2*c^2/fo - fo*norm(u_dot)^2 )+1/(c*r(i))*( 2*(hat_f(i)-fo)*c^2/fo*Jf_u(i,:) );
        Jfdot_fo(i,1:K) = 2/(c*r(i))*( ((hat_f(i)-fo)*c^2)/fo*Jf_udot(i,:) - fo*u_dot' );
        Jf_fo(i) = hat_f(i)/fo ;
        Jfdot_u(i,1) = hat_f_dot(i)/fo;
    end
    Jm_theta = [Jf_u,Jf_udot,Jf_fo;Jfdot_udot,Jfdot_fo,Jfdot_u];
    delta_theta = 0.5*inv(Jm_theta'*inv(Q)*Jm_theta)*Jm_theta'*inv(Q)*([hat_f;hat_f_dot]-[f;f_dot]);
    theta = theta - delta_theta;
    if norm(delta_theta)<1e-6
        break;
    end
end

end

