function [u,u_dot] = DSFDDLoc_SDP_Knownfo(f,df,c,fo,S,Q)
% [u,u_dot] = DSFDDLoc_SDP_Knownfo(f,df,c,fo,S,Q)
%
% SDP solution for DSF and DD localization when the center frequency is known.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement
% c:            scalar, signal propagation speed
% fo:           scalar, center frequency
% S:            (3 x M), sensor poition matrix
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
%
% Output:
% u:         (3 x 1), object position estimate 
% u_dot:     (3 x 1), object velocity estimate .
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors
Q = Q/Q(1,1);    % scaling the covariance matrix
p = (f-fo)*c/fo;
q = df*c/fo;
h1 = [zeros(M,1);p.^2];
G1 = [diag(p),-S',ones(M,1),zeros(M,1);
      diag(q),zeros(M,K+1),ones(M,1)];
G1 = [G1,zeros(2*M,K)];
Q1 = eye(2*M);

for iii=1:1:2
    F = [G1'/Q1*G1, -G1'/Q1*h1; -h1'/Q1*G1,1];
    L = M+2*K+2;
    
    % - solve SDP by cvx
    cvx_clear
    cvx_begin sdp quiet;
    cvx_solver sedumi
    cvx_precision best
    variable Y(M+2*K+2,M+2*K+2) symmetric;
    variable y(M+2*K+2);
    minimize( trace(F*[Y y;y' 1]) )
    subject to
    [Y y;y' 1]==semidefinite(M+K+2+K+1);
    for i =1:M
        Y(i,i)-S(:,i)'*S(:,i) + 2*S(:,i)'*y(M+K+3:M+2*K+2) - trace(Y(M+K+3:M+2*K+2,M+K+3:M+2*K+2))==0;
        Y(i,i)>=0;
        Y(i,i)+y(M+K+2)>=2*abs(y(M+K+1)-S(:,i)'*y(M+1:M+K));
        y(i)>=0;
        for j = i+1:1:M
            Y(i,j)>=abs( S(:,i)'*S(:,j) - ( S(:,i)+S(:,j) )'*y(M+K+3:M+2*K+2) + trace(Y(M+K+3:M+2*K+2,M+K+3:M+2*K+2)) );
        end
        Y(i,M+K+2)>=0;
    end
    y(M+K+1)-trace(Y(M+1:M+K,M+K+3:M+2*K+2))==0;
    y(M+K+2)-trace(Y(M+1:M+K,M+1:M+K))==0;
    trace(Y(M+K+3:M+2*K+2,M+K+3:M+2*K+2))+y(M+K+2)-2*abs(y(M+K+1))>=0;
    cvx_end

    % - find the object position and velocity estimate from SDP solution y and Y.
    for k = 1:1:L+1+L
        if k == L+1
            uu(:,k) = y(M+K+2+1:M+K+2+K);
            uu_dot(:,k) = y(M+1:M+K);
        elseif k<L+1
            uu(:,k) = Y(M+K+2+1:M+K+2+K,k)/(y(k));
            uu_dot(:,k) = Y(M+1:M+K,k)/y(k);
        else
            uu(:,k) = Y(M+K+2+1:M+K+2+K,k-L-1)/(sign(y(k-L-1))*sqrt(Y(k-L-1,k-L-1)));
            uu_dot(:,k) = Y(M+1:M+K,k-L-1)/(sign(y(k-L-1))*sqrt(Y(k-L-1,k-L-1)));
        end
    end
    Minval = Inf;
    for k1 = 1:1:2*L+1
        for k2 = 1:1:2*L+1
            u = uu(:,k1);
            u_dot = uu_dot(:,k2);
            r_hat = zeros(M,1);
            m_hat = zeros(2*M,1);
            for i = 1:1:M
                r_hat(i) = norm(u - S(:,i));
                m_hat(i) = fo*( 1 - (u-S(:,i))'*u_dot/(c*r_hat(i)) ) ;
                m_hat(M+i) =  ( ( ((m_hat(i)-fo)*c )^2 )/fo -fo*norm(u_dot)^2 )/(c*r_hat(i));
            end
            ObjFunc= (m_hat-[f;df])'/Q*(m_hat-[f;df]);
            if ObjFunc<Minval
                Idx_u = k1;
                Idx_udot = k2;
                Minval = ObjFunc;
            end
        end
    end
    u = uu(:,Idx_u);      % object position estimate
    u_dot = uu_dot(:,Idx_udot); % object velocity estimate

    % - update the weigting matrix
    for i =1:1:M
        r(i) = norm(u-S(:,i));
        B1(i,i) = -r(i);
        B1(i+M,i) = -2*(u-S(:,i))'*u_dot/r(i);
        B1(i+M,M+i) = -r(i);
    end
    Q1 = B1*Q*B1';
end

end
