function [u,u_dot,fo] = DSFDDLoc_SDP_Unknownfo(f,df,c,S,Q)
% [u,u_dot,fo] = DSFDDLoc_SDP_Unknownfo(f,df,c,S,Q)
%
% SDP solution for DSF and DD localization when the center frequency is unknown.
%
% The routine works for 2-D and 3-D scenarios
%
% Input:
% f:            (M x 1), DSF measurement, where M =  number of sensors
% f_dot:        (M x 1), DD measurement
% c:            scalar, signal propagation speed
% S:            (3 x M), sensor poition matrix
% Q:            (2M x 2M), covariance matrix of DSF and DD measurements
%
% Output:
% u:            (3 x 1), object position estimate 
% u_dot:        (3 x 1), object velocity estimate
% fo:           scalar, center frequency estimate
%
% Reference: X. Ke and K. C. Ho, "Localization by Doppler derivatives and 
% Doppler-shifted frequencies," IEEE Trans. Signal Process., vol. 72, pp. 2890-2904, 2024.
%
% X. Ke, K. C. Ho     7-21-2024
%
%       Copyright (C) 2024
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu

[K,M] = size(S); % K = localization dimension, M = number of sensors

% -- scaling the data --
scale_1 = max(diag(S'*S));
scale_2 = 10;
scale_3 = 10*scale_2;
S = S/scale_1;
c = c/scale_2;
f = f/scale_3;
df = scale_1/(scale_2*scale_3)*df;
Q = [Q(1:M,1:M),scale_1/scale_2*Q(1:M,M+1:2*M);
    scale_1/scale_2*Q(M+1:2*M,1:M),(scale_1/scale_2)^2*Q(M+1:2*M,M+1:2*M)];

tilde_fo = sum(f)/M;
tilde_f = f-tilde_fo;
d1 = [tilde_f.^3; tilde_f.^2];
Q1 = eye(2*M);

for i = 1:1:2
    H1 = [zeros(M,K), df.*S',      3*tilde_f.^2,   -3*tilde_f, tilde_f,     ones(M,1), -df,  zeros(M,M);
          zeros(M,K), zeros(M,K),    2*tilde_f, -ones(M,1), ones(M,1),  zeros(M,2),  diag(df)];
    Q1 = Q1/Q(1,1);
    L = 2*K+5+M;
    F = [H1'/Q1*H1, -H1'/Q1*d1; -d1'/Q1*H1,1];

    % -- solve SDP by cvx --
    cvx_clear
    cvx_solver sedumi
    cvx_precision best
    cvx_begin sdp quiet;
    variable Y(L,L) symmetric;
    variable y(L);
    minimize( trace(F*[Y,y;y',1]) )
    subject to
    [Y y;y' 1]==semidefinite(L+1);
    y(2*K+2) == Y(2*K+1,2*K+1);
    (tilde_fo/c)^2*y(2*K+3)+2*tilde_fo/c^2*Y(2*K+3,2*K+1)+Y(2*K+3,2*K+2)/c^2==trace(Y(K+1:2*K,K+1:2*K));
    y(2*K+4) == Y(2*K+1,2*K+2)-Y(2*K+1,2*K+3);
    tilde_fo/c*y(2*K+5)+Y(2*K+5,2*K+1)/c == trace(Y(1:K,K+1:2*K));
    Y(2*K+1,2*K+4)==Y(2*K+2,2*K+2)-Y(2*K+2,2*K+3);
    Y(2*K+2,2*K+3)>=0;
    trace(Y(1:K,1:K))+y(2*K+3)>=2*abs(y(2*K+5));
    for i =1:M
        Y(2*K+5+i,2*K+5+i)==S(:,i)'*S(:,i)/c^2*( tilde_fo^2+2*tilde_fo*y(2*K+1)+y(2*K+2) )-2*S(:,i)'*( tilde_fo/c*y(1:K)+Y(1:K,2*K+1)/c ) + trace(Y(1:K,1:K));
        Y(2*K+5+i,2*K+5+i)>=0;
        Y(2*K+5+i,2*K+2)>=0;
        Y(2*K+5+i,2*K+3)>=0;
        y(2*K+5+i)>=0;
        for j = i+1:1:M
            Y(2*K+5+i,2*K+5+j)>= abs(S(:,i)'*S(:,j)/c^2*( tilde_fo^2+2*tilde_fo*y(2*K+1)+y(2*K+2) )-( S(:,i)+S(:,j) )'*( tilde_fo/c*y(1:K)+Y(1:K,2*K+1)/c ) + trace(Y(1:K,1:K)));
        end
    end
    cvx_end

    % -- find the object position and velocity estimate from SDP solution y and Y --
    [AA,BB] = eig(Y);
    Y = BB(end,end)*AA(:,end)*AA(:,end)';
    for k = 1:1:L+1+L
        if k == L+1
            fo = tilde_fo+y(2*K+1);
            uu(:,k) = y(1:K)*c/fo;
            uu_dot(:,k) = y(K+1:2*K)*(c/fo)^2;
        elseif k<L+1
            fo = tilde_fo+Y(2*K+1,k)/y(k);
            uu(:,k) = Y(1:K,k)*c/(y(k)*fo);
            uu_dot(:,k) = Y(K+1:2*K,k)/y(k)*((c/fo)^2);
        else
            fo = tilde_fo+Y(2*K+1,k-L-1)/( sign(y(k-L-1))*sqrt(Y(k-L-1,k-L-1)) );
            uu(:,k) = Y(1:K,k-L-1)*c/(sign(y(k-L-1))*sqrt(Y(k-L-1,k-L-1))*fo);
            uu_dot(:,k) = Y(K+1:2*K,k-L-1)/(sign(y(k-L-1))*sqrt(Y(k-L-1,k-L-1)))*((c/fo)^2);
        end
        ffo(k) = fo;
        r_hat = zeros(M,1);
        m_hat = zeros(2*M,1);
        for i = 1:1:M
            r_hat(i) = norm(uu(:,k) - S(:,i));
            m_hat(i) = fo*( 1 - (uu(:,k)-S(:,i))'*uu_dot(:,k)/(c*r_hat(i)) ) ;
            m_hat(M+i) =  ( ( ((m_hat(i)-fo)*c )^2 )/fo -fo*norm(uu_dot(:,k))^2 )/(c*r_hat(i));
        end
        ObjFunc(k) = (m_hat-[f;df])'/Q*(m_hat-[f;df]);
    end    
    [~,Idx_BestSol] = min(ObjFunc);
    u_dot = uu_dot(:,Idx_BestSol);
    u = uu(:,Idx_BestSol);
    fo = ffo(Idx_BestSol);
    
    % - update the weigting matrix --
    for i = 1:1:M
        r_hat(i) = norm(u - S(:,i));
        f_hat(i) = fo*( 1 - (u-S(:,i))'*u_dot/(c*r_hat(i)) ) ;
    end
    check_C1 = [diag(3*(f_hat-fo).^2-(fo/c)^2*u_dot'*u_dot),diag( (fo/c)^2*(u-S)'*u_dot );
                2*diag(f_hat-fo),diag(-fo/c*r_hat)];
    Q1 = check_C1*Q*check_C1';
    Q1 = Q1/Q(1,1);
end
u_dot = uu_dot(:,Idx_BestSol)*scale_2;
u = uu(:,Idx_BestSol)*scale_1;
fo = ffo(Idx_BestSol)*scale_3;

end